@@ -1,4 +1,8 @@
+from tenacity import retry, stop_after_attempt, wait_random_exponential, retry_if_result, retry_if_exception_type, \
+    RetryError, retry_any
+
 UPLOADED_FILE_GCS_PATH_FORMAT = "{year}/{month}/{day}/{hour}/{minute}/{job_uuid}/uploaded.{chunk_number}.csv"
+
 DELIVERED_SCHEMA = [
     dict(name='segment_uuid', type='STRING'),
     dict(name='internal_user_id', type='STRING'),
@@ -10,7 +14,7 @@ DELIVERED_SCHEMA = [
     dict(name='created_ts', type='TIMESTAMP'),
 ]
 
-INTEGRATION_LOG = [
+INTEGRATION_LOG_SCHEMA = [
     dict(name='integration_job_uuid', type='STRING'),
     dict(name='segment_uuid', type='STRING'),
     dict(name='file_pattern', type='STRING'),
@@ -32,7 +36,7 @@ def log_integration(rows):
     create_table(project_id=get_settings()['integration_log_project_id'],
                  table_name=get_settings()['integration_log_table'],
                  dataset_name=get_settings()['integration_log_dataset'],
-                 schema=INTEGRATION_LOG)
+                 schema=INTEGRATION_LOG_SCHEMA)
 
     create_topic(get_settings()['integration_log_topic'])
     errors = stream_records_to_bigquery(rows=rows, row_ids=None,
@@ -63,36 +67,38 @@ def compose_integration_logs(job_uuid, segments, file_pattern, time, id_types, o
 
 
 def deliver_data(job_uuid, bucket_name, file_pattern, platform_name, destination, file_headers, segments, id_types,
-                 operations,
-                 **kwargs):
-    from stratus_api.integrations import get_integration_function, get_integration_settings
+                 operations, chunk_start_number, start_time=None, **kwargs):
+    from stratus_api.integrations import get_integration_settings
     from stratus_api.integrations.chunks import create_file_chunks
     from datetime import datetime
     from stratus_api.core.settings import get_settings
+    from stratus_api.integrations.chunks import get_file_checkpoint
+    if start_time is None:
+        start_time = datetime.utcnow()
+    else:
+        start_time = datetime.utcfromtimestamp(start_time)
+    if '*' in file_pattern and get_file_checkpoint(file_pattern=file_pattern, job_uuid=job_uuid) is None:
+        rows = compose_integration_logs(job_uuid, segments, file_pattern, int(start_time.timestamp()), id_types, operations,
+                                        'started')
+        log_integration(rows)
 
-    start_time = datetime.utcnow()
-
-    rows = compose_integration_logs(job_uuid, segments, file_pattern, int(start_time.timestamp()), id_types, operations,
-                                    'started')
-
-    log_integration(rows)
-
-    distribute_file = get_integration_function(function_name='distribute_file')
     segment_mapping = {i['segment_uuid']: i for i in segments}
     for local_path, delivered_path, chunk_number, in create_file_chunks(
             bucket_name=bucket_name,
             file_pattern=file_pattern,
             integration_settings=get_integration_settings(),
             file_headers=file_headers,
-            job_uuid=job_uuid):
+            job_uuid=job_uuid,
+            chunk_number=chunk_start_number
+    ):
         chunk_start = datetime.utcnow()
 
         try:
-            success = distribute_file(
+            success = distribution_retry_wrapper(
                 destination=destination, platform_name=platform_name, segments=segment_mapping, local_path=local_path,
                 chunk_number=chunk_number, start_time=start_time, id_types=id_types, operations=operations
             )
-        except Exception as e:
+        except RetryError as e:
             success = False
             if get_settings()['environment'] not in {'qa', 'prod'}:
                 raise e
@@ -142,6 +148,22 @@ def log_chunk_delivery(success, start, destination, chunk_number, segments, job_
     )
 
 
+def distribution_failed(value):
+    return value is False
+
+
+@retry(stop=stop_after_attempt(3), reraise=False, wait=wait_random_exponential(multiplier=.5, max=30),
+       retry=retry_any(retry_if_result(distribution_failed), retry_if_exception_type(Exception)))
+def distribution_retry_wrapper(**kwargs):
+    from stratus_api.integrations import get_integration_function
+    distribute_file = get_integration_function(function_name='distribute_file')
+    return distribute_file(**kwargs)
+
+
+def distribution_retry_logger(retry_state):
+    pass
+
+
 def load_delivered_data(job_uuid, local_path, chunk_number):
     from stratus_api.storage.gcs import upload_file_to_gcs
     from datetime import datetime
@@ -156,7 +178,6 @@ def load_delivered_data(job_uuid, local_path, chunk_number):
             day=now.day,
             hour=now.hour,
             minute=now.minute
-
         )
     )
     return True
